import torch
import numpy as np 
import json 
import cv2
from bytetracker import BYTETracker

"""from PromptTrack import Object_detection 
from Object_detection import get_inference"""
from PromptTrack.detection_utils  import get_inference
from PromptTrack.utils import read_video_mot
from PromptTrack.utils import xyxy_to_xywh 
from PromptTrack.utils import xywh_to_xyxy


#from utils import read_video_mot
#sys.path.append('/home/sophie/aggression_detection')
"""# Get the current script's directory
current_dir = os.path.dirname(os.path.realpath(__file__))
# Get the grandparent directory's path
grandparent_dir = os.path.abspath(os.path.join(current_dir, "../"))
# Add the grandparent directory to sys.path
sys.path.append(grandparent_dir)

# Now you can perform a relative import
"""


class PromptTracker():
    def __init__(self):
        self.track_thresh= 0
        print("")
        
    def process_a_frame(self,frame, frame_id, prompt="pigs",nms_threshold=0.8):
        """u choisis ton modele de detection d'objet u met dedans 

        Args:
            frame (_type_): _description_
            frame_id (_type_): _description_

        Returns:
            _type_: _description_
        """
        return {frame_id: [get_inference(frame, prompt,nms_threshold)]}


    # Define a function for processing a single frame
    """def process_frame(self, frame, frame_id, result_queue):
        # Your frame processing code goes here
        # For example, you can perform some image processing on the frame
        print(frame_id, "\n")
        processed_frame = {frame_id: [get_inference(frame)]}
        result_queue.put(processed_frame)"""




    def detect_objects(self,  video_file, prompt="pigs",nms_threshold=0.8):
        processed_frames = []
        video_capture = cv2.VideoCapture(video_file)  # Replace with your video file path
        frame_id=0
        while True:
            ret, frame = video_capture.read()
            if not ret:
                break
            processed_frame = self.process_a_frame (frame, frame_id, prompt,nms_threshold) 
            processed_frames.append(processed_frame)
            frame_id+=1
            """if frame_id==1000:
                break"""
            print("detection", frame_id)

        def key_function(item):
            return list(item.keys())[0]

        # Use argsort with the custom key function to sort the array
        sorted_indices = np.argsort(np.array([key_function(item) for item in processed_frames]))
        sorted_processed_frames = np.array(processed_frames)[sorted_indices]
        sorted_processed_frames= sorted_processed_frames.tolist()
        
        
        object_detection_file =video_file.split(".mp4")[0]+'_object_detection.json'
        with open(object_detection_file, 'w') as file:
            print('dumping the file')
            json.dump(sorted_processed_frames , file)

    def process_mot(self, video_file, fixed_parc=True, nbr_items=15, track_thresh=0, match_thresh=1, frame_rate=6, track_buffer=10000, max_time_lost=20000):
        """
        ***remplacer par Bytetrack qui a été crée avec des librairies existantes

        Args:
            video_file (str, optional): _description_. Defaults to "/home/sophie/aggression_detection/annotated/2019_11_22/000010/color.mp4".
        """
        self.track_thresh = track_thresh
        self.match_thresh = match_thresh
        self.frame_rate=frame_rate
        self.track_buffer = track_buffer
        self.max_time_lost= max_time_lost+1
        if fixed_parc ==False:
                nbr_items =float("inf")
                
        object_detection_file =video_file.split(".mp4")[0]+'_object_detection.json'
        with open(object_detection_file, "r") as file:
            # Parse the JSON data
            sorted_processed_frames = json.load(file)
        
        
        
        bytetracker = BYTETracker(track_thresh=self.track_thresh, track_buffer=self.track_buffer, match_thresh=self.match_thresh, frame_rate=frame_rate)  # j'oblige le modèle à faire matcher à de id existant ***mais ca ne marche pas *** 
        bytetracker.max_time_lost = self.max_time_lost
        track_over_time={}
        #tracker = CentroidTracker(max_lost=180) # or IOUTracker(...), CentroidKF_Tracker(...), SORT(...)
        for detection_dictionnary  in sorted_processed_frames:
            frame_id = int(list(detection_dictionnary.keys())[0])
            detection_bboxes, detection_confidences, detection_class_ids = list(detection_dictionnary.values())[0][0]
            detection_with_score=[]
            for idx, box in enumerate(detection_bboxes):
                box = xywh_to_xyxy(box)
                box = list(box)
                box.append(detection_confidences[idx])
                box.append(torch.tensor(detection_class_ids[idx]))
                detection_with_score.append(np.array(box))
                
                
            #we remove id greater than  9 to make sure we keep only the 9 pigs in the parc 
            #######
            
            
            for track in bytetracker.tracked_stracks:
                    if track.track_id>nbr_items:
                        bytetracker.tracked_stracks.remove(track)
                

            
            
            #########
            if len(detection_bboxes)!=0:
                detection_with_score =torch.tensor(detection_with_score)
                online_targets = bytetracker.update( detection_with_score, [1,1])
                #output_tracks = tracker.update(detection_bboxes, detection_confidences, detection_class_ids)
                track_over_time[frame_id]={}
                for track in online_targets : # idx, track in enumerate(detection_bboxes): # output_tracks:
                    #track=[0,idx,track[0], track[1], track[2], track[3]]
                    #frame, id, bb_left, bb_top, bb_width, bb_height, confidence, x, y, z = track
                    #assert len(track) == 10
                    #print(track)
                    track_id = track[4]
                    if track_id<nbr_items:
                        x, y, w, h =  xyxy_to_xywh(track[:4])
                        #break
                        #exit(0)
                        track_over_time[frame_id][track_id] = {"bbox":{'x':x, 'y':y, 'width':w, 'height':h} }
            else:
                track_over_time[frame_id]={}
                
        tracking_file =video_file.split(".mp4")[0]+'_mot_.json'
        with open(tracking_file, 'w') as file:
            json.dump(track_over_time , file)

    
    
    

    def read_video_with_mot(self,video_file):
        mot_file =video_file.split(".mp4")[0]+'_mot_.json'
        with open(mot_file, "r") as file:
            # Parse the JSON data
            tracks = json.load(file)    
        read_video_mot(video_file, tracks)

    #video_file = "/home/sophie/aggression_detection/annotated/2019_11_22/000010/color.mp4"

    


import torch
print(torch.cuda.is_available())
from PIL import Image
import requests
import torchvision.transforms as T
import matplotlib.pyplot as plt
from collections import defaultdict
import torch.nn.functional as F
import numpy as np
from skimage.measure import find_contours

from matplotlib import patches,  lines
from matplotlib.patches import Polygon
from nms import nms , malisiewicz, fast

torch.set_grad_enabled(False)




# standard PyTorch mean-std input image normalization
transform = T.Compose([
    T.Resize(800),
    T.ToTensor(),
    T.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
])

# for output bounding box post-processing
def box_cxcywh_to_xyxy(x):
    x_c, y_c, w, h = x.unbind(1)
    b = [(x_c - 0.5 * w), (y_c - 0.5 * h),
         (x_c + 0.5 * w), (y_c + 0.5 * h)]
    return torch.stack(b, dim=1)

def rescale_bboxes(out_bbox, size):
    img_w, img_h = size
    b = box_cxcywh_to_xyxy(out_bbox)
    b = b * torch.tensor([img_w, img_h, img_w, img_h], dtype=torch.float32)
    return b



# colors for visualization
COLORS = [[0.000, 0.447, 0.741], [0.850, 0.325, 0.098], [0.929, 0.694, 0.125],
          [0.494, 0.184, 0.556], [0.466, 0.674, 0.188], [0.301, 0.745, 0.933]]

def apply_mask(image, mask, color, alpha=0.5):
    """Apply the given mask to the image.
    """
    for c in range(3):
        image[:, :, c] = np.where(mask == 1,
                                  image[:, :, c] *
                                  (1 - alpha) + alpha * color[c] * 255,
                                  image[:, :, c])
    return image

def plot_results(pil_img, scores, boxes, labels, masks=None):
    plt.figure(figsize=(16,10))
    np_image = np.array(pil_img)
    ax = plt.gca()
    colors = COLORS * 100
    if masks is None:
      masks = [None for _ in range(len(scores))]
    assert len(scores) == len(boxes) == len(labels) == len(masks)
    for s, (xmin, ymin, xmax, ymax), l, mask, c in zip(scores, boxes.tolist(), labels, masks, colors):
        ax.add_patch(plt.Rectangle((xmin, ymin), xmax - xmin, ymax - ymin,
                                   fill=False, color=c, linewidth=3))
        text = f'{l}: {s:0.2f}'
        ax.text(xmin, ymin, text, fontsize=15, bbox=dict(facecolor='white', alpha=0.8))

        if mask is None:
          continue
        np_image = apply_mask(np_image, mask, c)

        padded_mask = np.zeros((mask.shape[0] + 2, mask.shape[1] + 2), dtype=np.uint8)
        padded_mask[1:-1, 1:-1] = mask
        contours = find_contours(padded_mask, 0.5)
        for verts in contours:
          # Subtract the padding and flip (y, x) to (x, y)
          verts = np.fliplr(verts) - 1
          p = Polygon(verts, facecolor="none", edgecolor=c)
          ax.add_patch(p)


    plt.imshow(np_image)
    plt.axis('off')
    plt.show()


def add_res(results, ax, color='green'):
    #for tt in results.values():
    if True:
        bboxes = results['boxes']
        labels = results['labels']
        scores = results['scores']
        #keep = scores >= 0.0
        #bboxes = bboxes[keep].tolist()
        #labels = labels[keep].tolist()
        #scores = scores[keep].tolist()
    #print(torchvision.ops.box_iou(tt['boxes'].cpu().detach(), torch.as_tensor([[xmin, ymin, xmax, ymax]])))
    
    colors = ['purple', 'yellow', 'red', 'green', 'orange', 'pink']
    
    for i, (b, ll, ss) in enumerate(zip(bboxes, labels, scores)):
        ax.add_patch(plt.Rectangle((b[0], b[1]), b[2] - b[0], b[3] - b[1], fill=False, color=colors[i], linewidth=3))
        cls_name = ll if isinstance(ll,str) else CLASSES[ll]
        text = f'{cls_name}: {ss:.2f}'
        print(text)
        ax.text(b[0], b[1], text, fontsize=15, bbox=dict(facecolor='white', alpha=0.8))
        
        
        
model, postprocessor = torch.hub.load('ashkamath/mdetr:main', 'mdetr_efficientnetB5', pretrained=True, return_postprocessor=True, trust_repo=True)
if torch.cuda.is_available():
  model = model.cuda()
model.eval();




def get_inference(im, caption="pigs",nms_threshold=0.8):
  im=Image.fromarray(im)
  # mean-std normalize the input image (batch-size: 1)
  img = transform(im).unsqueeze(0)
  if torch.cuda.is_available():
    img = img.cuda()

  # propagate through the model
  memory_cache = model(img, [caption], encode_and_save=True)
  outputs = model(img, [caption], encode_and_save=False, memory_cache=memory_cache)
  
  # keep only predictions with 0.7+ confidence
  probas = 1 - outputs['pred_logits'].softmax(-1)[0, :, -1].cpu()
  keep = (probas > 0.9).cpu()
  
  bboxes_scaled1 = rescale_bboxes(outputs['pred_boxes'].cpu()[0, :], im.size)
  areas = np.array([(box[2]-box[0] )* (box[3]-box[1]) for box in bboxes_scaled1])
  
  #filtrage sur l'aire 
  """area_filtered = (8000< areas) & ( areas<90000)
  keep = keep &  area_filtered
  """
  ############NMS
  bboxes_scaled_tlw= [[int(box[0]),int(box[1]), int(box[2]-box[0]), int(box[3]-box[1]) ]   for box in bboxes_scaled1]
  best_keep_nms = nms.boxes(bboxes_scaled_tlw, probas, nms_algorithm=fast.nms, nms_threshold=nms_threshold)
  best_keep_nms = np.array( [ True if (idx in best_keep_nms) else False for idx, i in enumerate(keep) ] )
  
  keep =keep & best_keep_nms
  #print("***",keep, best_keep_nms)
  

  # convert boxes from [0; 1] to image scales
  bboxes_scaled = rescale_bboxes(outputs['pred_boxes'].cpu()[0, keep], im.size)
  
  # Extract the text spans predicted by each box
  positive_tokens = (outputs["pred_logits"].cpu()[0, keep].softmax(-1) > 0.1).nonzero().tolist()
  predicted_spans = defaultdict(str)
  for tok in positive_tokens:
    item, pos = tok
    if pos < 255:
        span = memory_cache["tokenized"].token_to_chars(0, pos)
        predicted_spans [item] += " " + caption[span.start:span.end]

  labels = [predicted_spans [k] for k in sorted(list(predicted_spans .keys()))]
  detection_bboxes = []
  detection_confidences = []
  detection_class_ids = []

  for idx, detection in enumerate(bboxes_scaled):
    if  True: #8000<(detection[2]-detection[0])*(detection[3]-detection[1])<40000: # on filtre l'aire des 
      detection =detection.tolist()
      detection = [detection[0], detection[1], detection[2]-detection[0], detection[3]-detection[1]]
      #detection = to_elements(detection)
      detection_bboxes.append(detection)
      detection_confidences .append(probas[keep][idx].item())
      detection_class_ids  .append(1) #(labels[idx])
  
  return detection_bboxes, detection_confidences,detection_class_ids
  #plot_results(im, probas[keep], bboxes_scaled, labels)



import numpy as np
import copy
import cv2 
import warnings
def xyxy_to_xywh(xyxy_bbox):
    x_min, y_min, x_max, y_max = xyxy_bbox
    width = x_max - x_min
    height = y_max - y_min
    return x_min, y_min, width, height
def xywh_to_xyxy(xywh_bbox):
    x, y, width, height = xywh_bbox
    x_min = x
    y_min = y
    x_max = x + width
    y_max = y + height
    return x_min, y_min, x_max, y_max

def iou(box1, box2):
    # Calculate the coordinates of the intersection rectangle
    x1 = max(box1[0], box2[0])
    y1 = max(box1[1], box2[1])
    x2 = min(box1[2], box2[2])
    y2 = min(box1[3], box2[3])

    # Check for non-overlapping boxes
    if x1 < x2 and y1 < y2:
        # Calculate the area of the intersection rectangle
        intersection_area = (x2 - x1) * (y2 - y1)

        # Calculate the area of the bounding boxes
        area_box1 = (box1[2] - box1[0]) * (box1[3] - box1[1])
        area_box2 = (box2[2] - box2[0]) * (box2[3] - box2[1])

        # Calculate the IoU
        iou = intersection_area / (area_box1 + area_box2 - intersection_area)
        return iou
    else:
        return 0.0
    
def read_box(bbox):
    return [bbox["x"], bbox["y"], bbox["width"], bbox["height"]]
def adherence(bbox1, bbox2):
    bbox1 =read_box(bbox1)
    bbox2 =read_box(bbox2)

    bbox1 = xywh_to_xyxy ( bbox1)
    bbox2 = xywh_to_xyxy ( bbox2)
    if iou (bbox1, bbox2)>0:
        return 1
    else:
        return 0 
    
def distance(bbox1, bbox2):
    bbox1 =read_box(bbox1)
    bbox2 =read_box(bbox2)
    point1 = np.array([bbox1[0]+bbox1[2]/2 , bbox1[1]+bbox1[3]/2])
    point2 = np.array([bbox2[0]+bbox2[2]/2 , bbox2[1]+bbox2[3]/2])
    """bbox1 = xywh_to_xyxy ( bbox1)
    bbox2 = xywh_to_xyxy ( bbox2)"""
    return np.linalg.norm(point1 - point2)
    

def get_global_rectangle(bbox1,bbox2):
    """get rectangle englobant les 2 bounding boxes 

    Args:
        bbox1 (_type_): _description_
        bbox2 (_type_): _description_

    Returns:
        _type_: _description_
    """
    rect1=[bbox1["x"], bbox1["y"], bbox1["width"]+bbox1["x"], bbox1["height"]+bbox1["y"]]
    rect2=[bbox2["x"], bbox2["y"], bbox2["width"]+bbox2["x"], bbox2["height"]+bbox2["y"]]
    min_x = min(rect1[0], rect2[0])
    min_y = min(rect1[1], rect2[1])
    max_x = max(rect1[2], rect2[2])
    max_y = max(rect1[3], rect2[3])

    # Return the bounding box as a tuple (x1, y1, x2, y2)
    bounding_box = (min_x, min_y, max_x -min_x, max_y-min_y)
    bounding_box ={"x":bounding_box[0],"y":bounding_box[1], "width":bounding_box[2], "height":bounding_box[3] }
    return bounding_box


"""def compute_acceleration():
    return 0"""

def get_centroid(bbox):
    bbox2=[bbox["x"], bbox["y"], bbox["width"], bbox["height"]]
    return [bbox2[0] + bbox2[2]/2, bbox2[1] + bbox2[3]/2, bbox["x"], bbox["y"], bbox["width"], bbox["height"]]

def sort_dictionnary(myDict):
    #we make sure frame id are integer 
    myDict = {int(k):v for k,v in myDict.items()}
    myKeys = list(myDict.keys())
    #myKeys = [int(i) for i in myKeys]
    myKeys.sort()
    sorted_dict = {str(i): myDict[i] for i in myKeys}
    return sorted_dict

def process_speed_acceleration(sparsed_track,pig_id,frame_id):
    """_summary_
    we assume the track is sparsed and ordered in term of frames, the track is provided in pig_id->frame_id 
    #previous_available_frame_id_for_track previous_frame_id
    Args:
        sparsed_track (_type_): _description_
        pig_id (_type_): _description_
        frame_id (_type_): _description_
    """
    if int(frame_id)>0:
        frame_id_idx = list(sparsed_track[pig_id].keys()).index(frame_id)
        try:
            previous_frame_id = list(sparsed_track[pig_id].keys())[frame_id_idx-1]
            previous_centroid = sparsed_track[pig_id][previous_frame_id]["centroid"]
        except:
            warnings.warn("a previous frame with this centroid doesn't exist")
            return sparsed_track
        delta_t = int(frame_id) - int(previous_frame_id)
        centroid = sparsed_track[pig_id][frame_id]["centroid"]
        
        delta_x =  centroid[0] - previous_centroid[0]
        delta_y =  centroid[1] - previous_centroid[1]
        sparsed_track[pig_id][frame_id]["speed"] = np.sqrt(delta_x**2 + delta_y**2) / delta_t
    if int(frame_id)>1:
        ###process acceleration
        frame_id_idx = list(sparsed_track[pig_id].keys()).index(frame_id)
        try:
            previous_frame_id = list(sparsed_track[pig_id].keys())[frame_id_idx-1]
            previous_speed = sparsed_track[pig_id][previous_frame_id]["speed"]
        except:
            warnings.warn("a previous frame with this speed doesn't exist")
            return sparsed_track
        
        delta_t = int(frame_id) - int(previous_frame_id)
        speed = sparsed_track[pig_id][frame_id]["speed"]
       
        delta_speed =  speed - previous_speed
        #delta_y =  speed[1] - previous_speed[1]
        sparsed_track[pig_id][frame_id]["acceleration"] = delta_speed/delta_t#np.sqrt(delta_x**2 + delta_y**2) / delta_t
    
    return sparsed_track
    #for frame_id in sparsed_track[pig_id].keys()[]

def process_centroid_speed_acceleration(informations):
    transformed={}
    informations = sort_dictionnary(informations)
    for frame_id,frame_info in informations.items():
        for pig_id, pig_info in frame_info.items():
            if pig_id not in transformed.keys():
                transformed[pig_id]={}
            transformed[pig_id][frame_id]={"centroid":get_centroid(pig_info["bbox"])}
            transformed = process_speed_acceleration(transformed, pig_id, frame_id)

    for frame_id, frame_info in informations.items():
        for pig_id, pig_info in frame_info.items():
            try:
                speed = transformed[pig_id][frame_id]["speed"]
                rounded_speed= round(speed, 2)
            except:
                speed = None
                rounded_speed = " N"
            try:  
                acceleration = transformed[pig_id][frame_id]["acceleration"]
                rounded_acceleration = round(acceleration,  2)
            except:
                acceleration= None
                rounded_acceleration=" N"
            informations[frame_id][pig_id].update({"speed":speed,  "acceleration":acceleration , "behaviour": "v:"+str(rounded_speed)+" acc:"+str(rounded_acceleration)})

    return informations
    max_time = max([int(i) for i in informations.keys()])
    assert max_time == list(informations.keys())[-1]
    time_range_with_missing = np.arange(0,max_time+1)
    for pig_id, pig_over_time_centroid in transformed.items():
        #à partir de python 3.7 les dictionnaire sont garanties d'être lu dans l'ordre d'insertion
        pig_time= np.array(list(pig_over_time_centroid.keys()))
        pig_over_time_centroid = np.array(list(pig_over_time_centroid.values()))
        
        # missing_time = [i for i in time_range_with_missing if i not in pig_time]
        x_with_interpolated = np.interp(time_range_with_missing, pig_time, pig_over_time_centroid[:,0])
        y_with_interpolated = np.interp(time_range_with_missing, pig_time, pig_over_time_centroid[:,1])
        
        xmin_with_interpolated = np.interp(time_range_with_missing, pig_time, pig_over_time_centroid[:,2])
        ymin_with_interpolated = np.interp(time_range_with_missing, pig_time, pig_over_time_centroid[:,3])
        w_with_interpolated = np.interp(time_range_with_missing, pig_time, pig_over_time_centroid[:,4])
        h_with_interpolated = np.interp(time_range_with_missing, pig_time, pig_over_time_centroid[:,5])
        
        delta_t = np.diff(time_range_with_missing)#pig_time)
        delta_x = np.diff(x_with_interpolated)
        delta_y = np.diff(y_with_interpolated)
        
        speed = np.sqrt(delta_x**2 + delta_y**2) / delta_t
        acceleration = np.diff(speed) / delta_t[1:]
        
        for frame_id in time_range_with_missing[:-2]:
            if frame_id not in informations.keys():
                informations[frame_id]={}
            if pig_id not in informations[frame_id].keys():
                informations[frame_id][pig_id]={}
            
            informations[frame_id][pig_id].update({"bbox":{"x":xmin_with_interpolated[frame_id], "y":ymin_with_interpolated[frame_id], "width":w_with_interpolated[frame_id], "height":h_with_interpolated[frame_id]}, "centroid":[x_with_interpolated[frame_id], y_with_interpolated[frame_id]],"speed":speed[frame_id], "acceleration":acceleration[frame_id] })
      
    keys_to_remove = list(informations.keys())[-2:]
    new_informations= {key: informations[key] for key in informations if key not in keys_to_remove}
  
    return new_informations
        

def add_feature(informations, fps=6, aggression_duration= 10, acc_avg_th=5,acc_var_th=3, adh_index_percent=0.3, max_id=16 ):
    """
    dans le papier ils utilisent l'acceleration du groupe de bounding box comme feature de détection d'aggression
    je pense d'après les observations que c'est plutôt la variance dans l'acceleration, vitesse moyenne élevée, et un index d'adhesion élevé 
    """
    informations = {int(k):v for k,v in informations.items()}
    information_with= copy.deepcopy(informations)
    for frame_id, frame_information in informations.items():
        for pig_id_1, pig_info_1 in frame_information.items():
            for pig_id_2, pig_info_2 in frame_information.items():
                if pig_id_1!=pig_id_2:
                    join_pig_id=str(pig_id_1)+"and"+str(pig_id_2)
                    information_with[frame_id][join_pig_id] = {}
                    ad = adherence(pig_info_1["bbox"], pig_info_2["bbox"] )
                    
                    information_with[frame_id][join_pig_id]["adherence"]= ad
                    information_with[frame_id][join_pig_id]["distance"]= distance(pig_info_1["bbox"], pig_info_2["bbox"] )
                    information_with[frame_id][join_pig_id]["bbox"]= get_global_rectangle(pig_info_1["bbox"], pig_info_2["bbox"] )
                    #if ad: 
                    #    frame_information[join_pig_id]["adherence"]= ad
    information_with = process_centroid_speed_acceleration(information_with)
    
    
    ##we add acceleration and speed of the two pigs in a rectangle 
    for frame_id, frame_information in information_with.items():
        for pig_id, pig_info in frame_information.items():
            if "and" in pig_id:
                pig_1 = pig_id.split("and")[0]
                pig_2 = pig_id.split("and")[1]
                pig_info["acceleration1"] = information_with[frame_id][pig_1]["acceleration"]
                pig_info["acceleration2"] = information_with[frame_id][pig_2]["acceleration"]
                pig_info["speed1"] = information_with[frame_id][pig_1]["speed"]
                pig_info["speed2"] = information_with[frame_id][pig_2]["speed"]
    
    
    ############################compute aggression accordingly to chenchen 2017: #######################
    #1-Adhesion index of pair of rectangles: it's the percentage of frames where the pair of pigs have adhesion  in a fixed sequence of frames 
    # in our case we will consider 10 secondes (60 frames according to the pigbehavior dataset) as the lenght of each sequence 
    
    sequence_lenght = aggression_duration*fps
    list_of_pig_id=[str(float(i)) for i in range(1,max_id)] + [str(float(i))+"and"+str(float(j)) for i in range(1,max_id) for j in range(1,max_id)  if i!=j]
    key_list= { "distance":None, "acceleration":None, "acceleration1":None, "acceleration2":None,"speed1":None, "speed2":None, "speed":None,"adherence":None}# "bbox":None,
    transformed={}
    #informations = sort_dictionnary(information_with)
    behaviours={}
    previous =0
    for frame_id,frame_info in information_with.items():
        behaviours[frame_id]={}
        for pig_id in list_of_pig_id: #, pig_info in frame_info.items():
            if "and" in pig_id:
                if pig_id not in transformed.keys():
                    transformed[pig_id]={}
                if pig_id not in information_with[frame_id]:
                    transformed[pig_id][frame_id] =copy.deepcopy(key_list)
                else:
                    transformed[pig_id][frame_id]= copy.deepcopy(information_with[frame_id][pig_id]) 
                
                
        #if int(frame_id)<previous:
        #    continue
        previous=int(frame_id)+sequence_lenght
        
        for pig_id in list_of_pig_id: #, pig_info in frame_info.items():
            if "and" in pig_id:
                """if pig_id not in transformed.keys():
                    transformed[pig_id]={}
                if pig_id not in information_with[frame_id]:
                    transformed[pig_id][frame_id] =copy.deepcopy(key_list)
                else:
                    transformed[pig_id][frame_id]= copy.deepcopy(information_with[frame_id][pig_id]) 
                
                for key in key_list.keys() : #information_with[frame_id][pig_id].keys():
                    #if True: #key!="bbox":
                    #transformed[pig_id][frame_id][key+"-cumul"] = [transformed[pig_id][frame_id][key]]
                    if int(frame_id)==0:
                        transformed[pig_id][frame_id][key+"-cumul"] = []
                    #if int(frame_id)>0:
                    #    if frame_id in transformed[pig_id].keys():
                    #        transformed[pig_id][frame_id][key+"-cumul"] = transformed[pig_id][str(int(frame_id)-1)][key+"-cumul"] + [transformed[pig_id][frame_id][key]]
                    #    else:
                    else:"""
                           
                if int(frame_id)>sequence_lenght:
                    
                    for key in key_list.keys():
                        if "cumul" not in key  :
                            transformed[pig_id][frame_id][key+"-cumul"] = [transformed[pig_id][str(int(frame_i))][key] for frame_i in range(int(frame_id)-sequence_lenght+1,int(frame_id)+1 ) ] #+ [transformed[pig_id][frame_id][key]]
                            #transformed[pig_id][frame_id][key+"-cumul"] = transformed[pig_id][frame_id][key+"-cumul"][-sequence_lenght:]
                            tmp= [x for x in  transformed[pig_id][frame_id][key+"-cumul"][-sequence_lenght:] if x is not None]
                            transformed[pig_id][frame_id][key+"-cumul-avg"] = np.mean(tmp )
                            transformed[pig_id][frame_id][key+"-cumul-var"] = np.var(tmp , ddof=1)
                    adh_i = round(transformed[pig_id][frame_id]['adherence-cumul-avg'],1)
                    if adh_i> adh_index_percent:
                            behaviours[frame_id][pig_id]={}
                            for key in transformed[pig_id][frame_id].keys():
                                if 'cumul-' in key:
                                    behaviours[frame_id][pig_id][key] = transformed[pig_id][frame_id][key]
                            behaviours[frame_id][pig_id].update({'behaviour':"(i="+str(adh_i)+":"})
                                                                                            #+str(speed_avg)+","+str(acc_avg)+","+str(speed_var)+","+str(acc_var)+
                                                                                            #"-1:"+str(speed1_avg)+","+str(acc1_avg)+","+str(speed1_var)+","+str(acc1_var)+
                                                                                            #"-2:"+str(speed2_avg)+","+str(acc2_avg)+","+str(speed2_var)+","+str(acc2_var)+")" })
                    
                
    #return information_with
    #adh_index_th = adh_index_percent * sequence_lenght
    """behaviours={}
    for frame_id, frame_info in information_with.items():
        behaviours[frame_id]={}
        for pig_id, pig_info in frame_info.items():
            if 'and' in pig_id:
                information_with[frame_id][pig_id].update(transformed[pig_id][frame_id])  #transformed[pig_id][frame_id] #
                if int(frame_id)>sequence_lenght:
                    
                    adh_i = round(information_with[frame_id][pig_id]['adherence-cumul-avg'],1)
                    speed_avg = round(information_with[frame_id][pig_id]['speed-cumul-avg'],1)
                    acc_avg = round(information_with[frame_id][pig_id]['acceleration-cumul-avg'],1)
                    speed_var = round(information_with[frame_id][pig_id]['speed-cumul-var'],1)
                    acc_var = round(information_with[frame_id][pig_id]['acceleration-cumul-var'],1)
                    
                    speed1_avg = round(information_with[frame_id][pig_id]['speed1-cumul-avg'],1)
                    acc1_avg = round(information_with[frame_id][pig_id]['acceleration1-cumul-avg'],1)
                    speed1_var = round(information_with[frame_id][pig_id]['speed1-cumul-var'],1)
                    acc1_var = round(information_with[frame_id][pig_id]['acceleration1-cumul-var'],1)
                    
                    speed2_avg = round(information_with[frame_id][pig_id]['speed2-cumul-avg'],1)
                    acc2_avg = round(information_with[frame_id][pig_id]['acceleration2-cumul-avg'],1)
                    speed2_var = round(information_with[frame_id][pig_id]['speed2-cumul-var'],1)
                    acc2_var = round(information_with[frame_id][pig_id]['acceleration2-cumul-var'],1)
                    
                    if adh_i> adh_index_percent:
                        behaviours[frame_id][pig_id]={}
                        for key in information_with[frame_id][pig_id].keys():
                            if 'cumul-' in key:
                                behaviours[frame_id][pig_id][key] = information_with[frame_id][pig_id][key]
                        behaviours[frame_id][pig_id].update({'behaviour':"(i="+str(adh_i)+":"})
                                                                                        #+str(speed_avg)+","+str(acc_avg)+","+str(speed_var)+","+str(acc_var)+
                                                                                        #"-1:"+str(speed1_avg)+","+str(acc1_avg)+","+str(speed1_var)+","+str(acc1_var)+
                                                                                        #"-2:"+str(speed2_avg)+","+str(acc2_avg)+","+str(speed2_var)+","+str(acc2_var)+")" })
                    #else:
                    #   behaviours[frame_id][pig_id].update({"behavioue": '<index threshold'}) 
    """
    return behaviours# information_with
        
    

def add_rectangle_on_frame(frame, coordinates, title):
    """_summary_

    Args:
        frame (_type_): _description_
        coordinates (_type_): in x0, y0, w, h 
        text (_type_): _description_

    Returns:
        _type_: _description_
    """
    # Add a rectangle to the frame
    rectangle_color = (0, 255, 0)  # Green color
    rectangle_thickness = 2
    coordinates = [int(i) for i in coordinates]
    cv2.rectangle(frame, (coordinates[0], coordinates[1]), (coordinates[0]+coordinates[2], coordinates[1]+coordinates[3]), rectangle_color, rectangle_thickness)

    # Add a title to the frame
    
    title_position = (coordinates[0], coordinates[1]-2)
    title_font = 0# cv2.FONT_HERSHEY_SIMPLEX
    title_font_scale = 0.6  
    title_color = (0, 255, 0)  # Green color
    cv2.putText(frame, title, title_position, title_font, title_font_scale, title_color, rectangle_thickness, cv2.LINE_AA)
    return frame 

def read_video_mot(video_file, annotations):
    annotated_video_file = video_file.split(".mp4")[0]+'_with_id.mp4'
    cap = cv2.VideoCapture(video_file)
    # Check if the video file was opened successfully
    frame_width = int(cap.get(3))
    frame_height = int(cap.get(4))
    fps = int(cap.get(cv2.CAP_PROP_FPS))
    fourcc = cv2.VideoWriter_fourcc(*'XVID')  # You can change the codec as needed
    out = cv2.VideoWriter(annotated_video_file, fourcc, 6, (frame_width, frame_height))

    if not cap.isOpened():
        print("Error: Could not open video file.")
        exit()

    frame_id=0
    while True:
        # Read a frame from the video
        ret, frame = cap.read()
        
        # Check if the video has ended
        if not ret :
            break
        
        if frame_id in annotations.keys() or str(frame_id) in annotations.keys():
            print("frame_id",frame_id)
            for pig_id,pig  in annotations[str(frame_id)].items():
                if True: #pig["isGroundTruth"]==True and pig["behaviour"]=="fight":
                    coordinates = [pig["bbox"]["x"],pig["bbox"]["y"], pig["bbox"]["width"], pig["bbox"]["height"]]
                    if 'behaviour' in pig.keys():
                        behaviour = pig["behaviour"]
                    else:
                        behaviour=""
                    frame= add_rectangle_on_frame(frame, coordinates, str(pig_id)+" :"+behaviour)
        
            out.write(frame)
        frame_id+=1
        # Process the frame (you can do whatever you want with it here)
        # For example, you can display the frame
        # cv2.imshow('Video', frame)
        # Exit when 'q' key is pressed
        # if cv2.waitKey(1) & 0xFF == ord('q'):
        #   break
    # Release the video capture object and close any windows
    #cap.release()
    out.release()

    cv2.destroyAllWindows()

        